"""
Command-line interface for RedXSecureUpdater
Supports on-demand updates and scheduled updates
"""
import os
import sys
import argparse
import logging
import datetime
from pathlib import Path
from typing import Optional
import schedule
import time
from .core import RedXSecureUpdater

def parse_time(time_str: str) -> Optional[datetime.datetime]:
    """Parse time string in format HH:MM AM/PM"""
    try:
        return datetime.datetime.strptime(time_str, "%I:%M %p").time()
    except ValueError:
        return None

class RedXSecureUpdateCLI:
    def __init__(self):
        self.updater = None
        self.scheduled_time = None
        self.update_message = None

    def initialize(self, directory: str, remote_url: str = None, log_level: int = logging.INFO):
        """Initialize the updater"""
        self.updater = RedXSecureUpdater(
            directory=directory,
            remote_url=remote_url,
            watch=False,  # Don't watch continuously, only check when requested
            log_level=log_level
        )

    def schedule_update(self, time_str: str):
        """Schedule update for specific time"""
        update_time = parse_time(time_str)
        if not update_time:
            print(f"Invalid time format: {time_str}. Use format like '3:45 PM'")
            return False

        schedule.every().day.at(update_time.strftime("%H:%M")).do(self.check_and_update)
        self.scheduled_time = update_time
        print(f"Update scheduled for {time_str}")
        return True

    def check_and_update(self):
        """Check for and apply updates"""
        if not self.updater:
            print("Updater not initialized")
            return False

        try:
            print("Checking for updates...")
            if self.updater.check_for_updates():
                if self.update_message:
                    print(f"\nUpdate Message: {self.update_message}")
                print("Updates available, applying...")
                return True
            else:
                print("No updates available")
                return False

        except Exception as e:
            print(f"Error during update: {e}")
            return False

def main():
    """Main CLI entrypoint"""
    parser = argparse.ArgumentParser(description="RedXSecureUpdater CLI")

    parser.add_argument(
        "-d", "--directory",
        default=".",
        help="Application directory (default: current directory)"
    )

    parser.add_argument(
        "-r", "--remote-url",
        help="Remote update server URL"
    )

    parser.add_argument(
        "-m", "--message",
        help="Update message to display"
    )

    parser.add_argument(
        "-t", "--time",
        help="Schedule update time (format: HH:MM AM/PM, e.g., '3:45 PM')"
    )

    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Enable verbose logging"
    )

    args = parser.parse_args()

    # Setup logging
    log_level = logging.DEBUG if args.verbose else logging.INFO
    logging.basicConfig(
        level=log_level,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    )

    try:
        cli = RedXSecureUpdateCLI()
        cli.initialize(
            directory=args.directory,
            remote_url=args.remote_url,
            log_level=log_level
        )

        if args.message:
            cli.update_message = args.message

        if args.time:
            if not cli.schedule_update(args.time):
                sys.exit(1)
            print(f"\nRedXSecureUpdater scheduled for {args.time}")
            print("Waiting for scheduled time...")
            while True:
                schedule.run_pending()
                time.sleep(1)
        else:
            # Run immediate update check
            if cli.check_and_update():
                print("Update completed successfully")
            else:
                print("Update not needed or failed")
            sys.exit(0)

    except KeyboardInterrupt:
        print("\nUpdate check cancelled")
        sys.exit(0)
    except Exception as e:
        print(f"Error: {str(e)}", file=sys.stderr)
        sys.exit(1)

if __name__ == "__main__":
    main()