"""
Compression functionality for RedXSecureUpdater
"""
import gzip
import logging
import tempfile
from pathlib import Path
from .utils import get_file_size, format_size

class Compressor:
    def __init__(self, compression_level=6):
        self.compression_level = compression_level
        self.logger = logging.getLogger(__name__)
        
    def compress_file(self, file_path: Path) -> Path:
        """Compress a file using gzip"""
        try:
            original_size = get_file_size(file_path)
            compressed_path = Path(tempfile.gettempdir()) / f"{file_path.name}.gz"
            
            with open(file_path, 'rb') as f_in:
                with gzip.open(compressed_path, 'wb', 
                             compresslevel=self.compression_level) as f_out:
                    f_out.write(f_in.read())
                    
            compressed_size = get_file_size(compressed_path)
            ratio = (1 - compressed_size / original_size) * 100
            
            self.logger.info(
                f"Compressed {file_path.name}: "
                f"{format_size(original_size)} → {format_size(compressed_size)} "
                f"({ratio:.1f}% reduction)"
            )
            
            return compressed_path
        except Exception as e:
            self.logger.error(f"Compression failed for {file_path}: {str(e)}")
            return None
            
    def decompress_file(self, file_path: Path, output_path: Path) -> bool:
        """Decompress a gzipped file"""
        try:
            with gzip.open(file_path, 'rb') as f_in:
                with open(output_path, 'wb') as f_out:
                    f_out.write(f_in.read())
                    
            self.logger.info(f"Decompressed {file_path.name} to {output_path}")
            return True
        except Exception as e:
            self.logger.error(f"Decompression failed for {file_path}: {str(e)}")
            return False
