"""
Core functionality for RedXSecureUpdater with real-time monitoring
"""
from pathlib import Path
from typing import Optional, Tuple
from datetime import datetime, timedelta
import os
import sys
import json
import logging
import subprocess
import time
import requests
try:
    from rich.console import Console
except ImportError:
    # Fallback for environments without rich
    class Console:
        def __init__(self):
            pass
        def print(self, message):
            print(message.replace('[blue]', '').replace('[/blue]', '')
                  .replace('[red]', '').replace('[/red]', '')
                  .replace('[yellow]', '').replace('[/yellow]', '')
                  .replace('[green]', '').replace('[/green]', ''))

from .monitor import FileMonitor
from .version_control import VersionControl
from .remote_updater import RemoteUpdater

class RedXSecureUpdater:
    def __init__(
        self, 
        directory: str,
        remote_url: Optional[str] = None,
        watch: bool = False,
        log_level: int = logging.INFO,
        update_message: Optional[str] = None,
        schedule_time: Optional[str] = None,
        version: Optional[str] = None,
        app_command: Optional[str] = None
    ):
        """Initialize RedXSecureUpdater with enhanced version control"""
        self.logger = logging.getLogger("RedXSecureUpdater")
        self.logger.setLevel(log_level)
        self.logger.handlers.clear()

        handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)

        # Setup paths and configuration
        self.directory = Path(directory).resolve()
        self.state_dir = self.directory / '.redx'
        self.state_file = self.state_dir / 'redx_state.json'
        self.app_command = app_command

        # Initialize components
        self._setup_directories()
        self.version_control = VersionControl(str(self.directory))

        # Update configuration
        self.update_message = update_message
        self._schedule_time = None  # Internal parsed schedule time
        self.schedule_time_str = schedule_time  # Original time string
        self.console = Console()
        self.is_running = False
        self._pending_version = version

        # Set version if provided
        if version:
            self.logger.info(f"Setting version to: {version}")
            try:
                if not self.version_control.set_version(version):
                    self.logger.error("Failed to set initial version")
                    raise ValueError(f"Failed to set version: {version}")
            except ValueError as ve:
                self.logger.error(f"Invalid version format: {str(ve)}")
                raise

        # Parse schedule time if provided
        if schedule_time:
            if not self._parse_schedule_time(schedule_time):
                raise ValueError(f"Invalid schedule time format: {schedule_time}")

        self.remote = RemoteUpdater(str(self.directory), remote_url) if remote_url else None
        self.monitor = FileMonitor(str(self.directory)) if watch else None

    def _setup_directories(self) -> None:
        """Create necessary directories for the updater"""
        try:
            # List of required directories
            directories = [
                self.directory,
                self.state_dir,
                self.directory / '.redx_backups',
                self.directory / '.redx_history'
            ]

            # Create each directory with proper permissions
            for dir_path in directories:
                dir_path.mkdir(parents=True, exist_ok=True)
                if hasattr(os, 'chmod'):  # Only on Unix-like systems
                    os.chmod(str(dir_path), 0o755)

            self.logger.info("Successfully initialized directory structure")

        except Exception as e:
            self.logger.error(f"Failed to create directories: {str(e)}")
            raise RuntimeError(f"Directory setup failed: {str(e)}")

    def _restart_application(self):
        """Restart the application after update in a platform-independent way"""
        try:
            self.console.print("[yellow]Restarting application...[/yellow]")

            if self.app_command:
                # Use platform-independent subprocess call
                subprocess.Popen(
                    self.app_command,
                    shell=True,
                    start_new_session=True,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE
                )
                self.logger.info("Application restart initiated")
                return True
            return False

        except Exception as e:
            self.logger.error(f"Failed to restart application: {str(e)}")
            return False

    def _parse_schedule_time(self, time_str: str) -> bool:
        """Parse and validate schedule time with improved format handling"""
        try:
            # Try various time formats
            formats = [
                "%I:%M %p",  # 3:45 PM
                "%I:%M%p",   # 3:45PM
                "%I:%M%p",   # 3:45pm (case insensitive)
                "%H:%M",     # 15:45 (24-hour)
            ]

            time_str = time_str.strip().upper()

            for fmt in formats:
                try:
                    parsed_time = datetime.strptime(time_str, fmt)
                    # Convert to today's date with the specified time
                    today = datetime.now().replace(
                        hour=parsed_time.hour,
                        minute=parsed_time.minute,
                        second=0,
                        microsecond=0
                    )

                    # If the time has passed today, schedule for tomorrow
                    if today < datetime.now():
                        today += timedelta(days=1)

                    self._schedule_time = today
                    self.logger.info(f"Updates scheduled for {time_str}")
                    self.console.print(f"[blue]Update scheduled for {time_str}[/blue]")
                    return True
                except ValueError:
                    continue

            self.logger.error(f"Invalid time format: {time_str}")
            self.console.print(f"[red]Invalid time format: {time_str}[/red]")
            self.console.print("[yellow]Please use format like '3:45 PM' or '15:45'[/yellow]")
            return False

        except Exception as e:
            self.logger.error(f"Failed to parse schedule time: {str(e)}")
            return False

    def check_and_update(self) -> bool:
        """Check for and apply updates with improved version handling"""
        try:
            update_applied = False

            if self.update_message:
                self.console.print(f"\n[yellow]Update Message:[/yellow] {self.update_message}")

            current_version = self.version_control.get_version()
            self.logger.info(f"Current version: {current_version}")

            # First handle version update if specified
            if self._pending_version:
                try:
                    self.logger.info(f"Applying pending version update: {self._pending_version}")
                    if not self.version_control.set_version(self._pending_version):
                        self.console.print(f"[red]Failed to update version[/red]")
                        return False
                    self.console.print(f"[green]Version updated to {self._pending_version}[/green]")
                    update_applied = True
                except ValueError as ve:
                    self.console.print(f"[red]Version error: {str(ve)}[/red]")
                    return False

            self._close_ui_windows()
            self.console.print("[yellow]Checking for updates...[/yellow]")

            if self.remote and self.remote.check_updates():
                self.console.print("[yellow]Updates available, applying...[/yellow]")
                changes = self.remote.check_updates()
                if changes and self.remote.apply_updates(changes):
                    self.console.print("[green]Updates applied successfully[/green]")
                    update_applied = True

            if update_applied:
                if self.app_command:
                    return self._restart_application()
                return True

            self.console.print("[blue]No updates available[/blue]")
            return update_applied

        except Exception as e:
            self.logger.error(f"Update check failed: {str(e)}")
            self.console.print(f"[red]Update check failed: {str(e)}[/red]")
            return False

    def _close_ui_windows(self):
        """Close UI windows while keeping command prompt"""
        try:
            if os.name == 'nt':  # Windows
                subprocess.run(['taskkill', '/F', '/IM', 'pythonw.exe'], capture_output=True)
            else:  # Unix-like
                subprocess.run(['pkill', '-f', 'pythonw'], capture_output=True)
        except Exception as e:
            self.logger.error(f"Failed to close UI windows: {str(e)}")

    def run(self) -> bool:
        """Run the updater with improved scheduling and error handling"""
        try:
            self.is_running = True

            if self._schedule_time:
                self.console.print(f"[blue]Update scheduled for {self.schedule_time_str}[/blue]")

                while self.is_running:
                    now = datetime.now()
                    if now >= self._schedule_time:
                        result = self.check_and_update()
                        # Reset schedule time to next day if needed
                        self._schedule_time += timedelta(days=1)
                        return result

                    # Sleep until next minute
                    sleep_time = 60 - now.second
                    time.sleep(sleep_time)
            else:
                return self.check_and_update()

        except KeyboardInterrupt:
            self.console.print("\n[yellow]Update process cancelled[/yellow]")
            return False
        except Exception as e:
            self.logger.error(f"Error during update run: {str(e)}")
            self.console.print(f"[red]Error: {str(e)}[/red]")
            return False
        finally:
            self.stop()

    def stop(self):
        """Stop the updater"""
        if not self.is_running:
            return

        try:
            self.is_running = False
            if self.monitor:
                self.monitor.stop()
            self.logger.info("Updater stopped successfully")

        except Exception as e:
            self.logger.error(f"Error stopping updater: {str(e)}")

def RedXSecureUpdate(
    message: Optional[str] = None,
    schedule_time: Optional[str] = None,
    version: Optional[str] = None,
    app_command: Optional[str] = None,
    directory: str = "."
) -> bool:
    """
    Console command to run updates with optional message, schedule, and version

    Args:
        message: Optional update message to display
        schedule_time: Optional schedule time (format: "HH:MM AM/PM", e.g., "3:45 PM")
        version: Optional version to set (e.g., "1.0.1" or "Version 2.0")
        app_command: Optional command to restart application after update
        directory: Application directory (default: current directory)
    """
    # Clean up input strings
    if message:
        message = message.strip().strip("'\"")
    if schedule_time:
        schedule_time = schedule_time.strip().strip("'\"")
    if version:
        version = version.strip().strip("'\"")
    if app_command:
        app_command = app_command.strip().strip("'\"")
    if directory:
        directory = directory.strip().strip("'\"")

    try:
        updater = RedXSecureUpdater(
            directory=directory,
            update_message=message,
            schedule_time=schedule_time,
            version=version,
            app_command=app_command
        )

        return updater.run()
    except Exception as e:
        logging.error(f"Update failed: {str(e)}")
        return False