"""
Enhanced file system monitoring for RedXSecureUpdater
"""
import os
import logging
import threading
from pathlib import Path
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler, FileSystemEvent
from typing import Optional, Callable

class FileMonitor:
    def __init__(self, directory: str):
        self.directory = Path(directory).resolve()
        self.observer = Observer()
        self.logger = logging.getLogger(__name__)
        self.callback = None
        self.is_running = False
        self.event_lock = threading.Lock()
        self._ensure_directory()

    def _ensure_directory(self):
        """Ensure monitored directory exists"""
        try:
            if not self.directory.exists():
                self.directory.mkdir(parents=True, exist_ok=True)
                os.chmod(str(self.directory), 0o755)
            self.logger.info(f"Monitor initialized for directory: {self.directory}")
        except Exception as e:
            self.logger.error(f"Failed to initialize monitor directory: {e}")
            raise

    def set_callback(self, callback: Callable[[FileSystemEvent], None]):
        """Set callback for file events with thread safety"""
        with self.event_lock:
            self.callback = callback
            self.logger.debug("Callback function set")

    def start(self):
        """Start monitoring with enhanced error handling"""
        if self.is_running:
            return

        try:
            handler = FileChangeHandler(
                base_dir=self.directory,
                callback=self.callback,
                logger=self.logger,
                event_lock=self.event_lock
            )

            self.observer.schedule(
                handler,
                str(self.directory),
                recursive=True
            )

            self.observer.start()
            self.is_running = True
            self.logger.info(f"Started monitoring directory: {self.directory}")

        except Exception as e:
            self.logger.error(f"Failed to start monitoring: {e}")
            self.is_running = False
            raise

    def stop(self):
        """Stop monitoring with cleanup"""
        if not self.is_running:
            return

        try:
            if self.observer.is_alive():
                self.observer.stop()
                self.observer.join(timeout=5)

            self.is_running = False
            self.logger.info("Stopped file monitoring")

        except Exception as e:
            self.logger.error(f"Error stopping monitor: {e}")
            self.is_running = False

class FileChangeHandler(FileSystemEventHandler):
    def __init__(
        self,
        base_dir: Path,
        callback: Optional[Callable] = None,
        logger: Optional[logging.Logger] = None,
        event_lock: Optional[threading.Lock] = None
    ):
        self.base_dir = Path(base_dir).resolve()
        self.callback = callback
        self.logger = logger or logging.getLogger(__name__)
        self.event_lock = event_lock or threading.Lock()

    def on_any_event(self, event: FileSystemEvent):
        """Handle file system events with improved filtering"""
        if event.is_directory:
            return

        try:
            file_path = Path(event.src_path).resolve()

            # Convert to relative path for consistent handling
            try:
                rel_path = file_path.relative_to(self.base_dir)
            except ValueError:
                self.logger.warning(f"File {file_path} is outside monitored directory")
                return

            # Skip special paths with comprehensive checks
            if self._should_ignore_path(file_path):
                return

            self.logger.debug(f"Processing event for {rel_path}: {event.event_type}")

            # Thread-safe callback execution
            with self.event_lock:
                if self.callback:
                    self.callback(event)

        except Exception as e:
            self.logger.error(f"Error handling file event: {e}")

    def _should_ignore_path(self, file_path: Path) -> bool:
        """Enhanced path filtering"""
        try:
            # Convert to relative path for consistent checks
            try:
                rel_path = file_path.relative_to(self.base_dir)
            except ValueError:
                return True

            # Comprehensive ignore patterns
            ignore_patterns = [
                lambda p: any(part.startswith('.') for part in p.parts),
                lambda p: p.name.startswith('~'),
                lambda p: p.suffix in ['.pyc', '.pyo', '.pyd', '.tmp'],
                lambda p: '.git' in p.parts,
                lambda p: '__pycache__' in p.parts,
                lambda p: p.name.endswith('~'),
                lambda p: 'redx_state.json' in str(p),
                lambda p: '.redx_history' in str(p)
            ]

            return any(pattern(rel_path) for pattern in ignore_patterns)

        except Exception as e:
            self.logger.error(f"Error in path filtering for {file_path}: {e}")
            return True