"""
Executable packaging and update functionality for RedXSecureUpdater
"""
import os
import sys
import logging
import shutil
from pathlib import Path
from typing import Optional, List
import PyInstaller.__main__

class RedXPackager:
    def __init__(self, project_dir: str, main_script: str, output_dir: Optional[str] = None):
        """
        Initialize the packager
        
        Args:
            project_dir: Root directory of your project
            main_script: Path to your main Python script
            output_dir: Directory where the executable will be created
        """
        self.project_dir = Path(project_dir).resolve()
        self.main_script = self.project_dir / main_script
        self.output_dir = Path(output_dir or 'dist').resolve()
        self.logger = logging.getLogger('RedXSecureUpdater')

    def create_executable(self, icon_path: Optional[str] = None, 
                        additional_files: Optional[List[str]] = None,
                        hidden_imports: Optional[List[str]] = None) -> bool:
        """
        Create an executable from the project
        
        Args:
            icon_path: Path to .ico file for the executable
            additional_files: List of additional files to include
            hidden_imports: List of modules to include
        """
        try:
            # Prepare PyInstaller arguments
            args = [
                self.main_script.as_posix(),
                '--onefile',  # Create a single executable
                '--clean',    # Clean PyInstaller cache
                '--noconfirm',  # Replace output directory without asking
                f'--distpath={self.output_dir}',
                '--add-data', f'{self.project_dir / ".redx_updater"};.redx_updater'
            ]

            if icon_path:
                args.extend(['--icon', icon_path])

            # Add hidden imports
            if hidden_imports:
                for module in hidden_imports:
                    args.extend(['--hidden-import', module])

            # Add additional files
            if additional_files:
                for file in additional_files:
                    src = self.project_dir / file
                    if src.exists():
                        args.extend(['--add-data', f'{src};{file}'])

            # Run PyInstaller
            PyInstaller.__main__.run(args)
            self.logger.info(f"Created executable in {self.output_dir}")
            return True

        except Exception as e:
            self.logger.error(f"Failed to create executable: {e}")
            return False

    def update_executable(self, exe_path: Path) -> bool:
        """
        Update an existing executable with new files
        
        Args:
            exe_path: Path to the executable to update
        """
        try:
            # Create backup of the executable
            backup_path = exe_path.with_suffix('.bak')
            shutil.copy2(exe_path, backup_path)

            # Create new executable
            if self.create_executable():
                # Replace old executable with new one
                shutil.copy2(self.output_dir / exe_path.name, exe_path)
                backup_path.unlink()  # Remove backup
                self.logger.info("Successfully updated executable")
                return True

            # Restore from backup if update failed
            shutil.copy2(backup_path, exe_path)
            backup_path.unlink()
            self.logger.error("Failed to update executable, restored from backup")
            return False

        except Exception as e:
            self.logger.error(f"Error updating executable: {e}")
            if backup_path.exists():
                try:
                    shutil.copy2(backup_path, exe_path)
                    backup_path.unlink()
                    self.logger.info("Restored executable from backup")
                except Exception as restore_error:
                    self.logger.error(f"Failed to restore from backup: {restore_error}")
            return False
