"""
Remote update functionality for RedXSecureUpdater
"""
import os
import json
import logging
import requests
import shutil
from pathlib import Path
from urllib.parse import urlparse, unquote
from .utils import calculate_checksum

class RemoteUpdater:
    def __init__(self, base_dir, remote_url):
        self.base_dir = Path(base_dir).resolve()
        self.remote_url = remote_url
        self.logger = logging.getLogger(__name__)
        self._validate_url()
        self._initialize_manifest()

    def _validate_url(self):
        """Validate and normalize remote URL"""
        parsed = urlparse(self.remote_url)
        if parsed.scheme not in ('http', 'https', 'file'):
            raise ValueError(f"Unsupported URL scheme: {parsed.scheme}")

    def _initialize_manifest(self):
        """Initialize local manifest if it doesn't exist"""
        try:
            # Initialize local manifest
            manifest_path = self.base_dir / ".redx_manifest.json"
            initial_manifest = {
                "version": "0.0.0",
                "files": {}
            }

            manifest_path.parent.mkdir(parents=True, exist_ok=True)
            if not manifest_path.exists():
                # Atomic write using temporary file
                temp_path = manifest_path.with_suffix('.tmp')
                with open(temp_path, 'w') as f:
                    json.dump(initial_manifest, f, indent=2)
                temp_path.replace(manifest_path)
                self.logger.info("Initialized local manifest file")

            # Initialize remote manifest for file:// URLs
            if self.remote_url.startswith('file://'):
                remote_path = Path(unquote(urlparse(self.remote_url).path)).resolve()
                remote_manifest = remote_path / "manifest.json"

                remote_path.mkdir(parents=True, exist_ok=True)
                if not remote_manifest.exists():
                    # Atomic write using temporary file
                    temp_path = remote_manifest.with_suffix('.tmp')
                    with open(temp_path, 'w') as f:
                        json.dump(initial_manifest, f, indent=2)
                    temp_path.replace(remote_manifest)
                    self.logger.info("Initialized remote manifest file")

        except Exception as e:
            self.logger.error(f"Failed to initialize manifest: {str(e)}")
            raise

    def publish_update(self, file_path):
        """Publish local file to remote location"""
        try:
            if not self.remote_url.startswith('file://'):
                self.logger.error("Only file:// URLs are supported for publishing")
                return False

            local_path = (self.base_dir / file_path).resolve()
            if not local_path.exists():
                self.logger.error(f"Local file not found: {local_path}")
                return False

            remote_base = Path(unquote(urlparse(self.remote_url).path)).resolve()
            remote_path = remote_base / file_path

            # Create remote directory if needed
            remote_path.parent.mkdir(parents=True, exist_ok=True)

            # Copy file to remote location
            shutil.copy2(local_path, remote_path)

            # Update remote manifest
            checksum = calculate_checksum(local_path)
            if self._update_remote_manifest(file_path, checksum):
                self.logger.info(f"Published update for {file_path}")
                return True

            return False

        except Exception as e:
            self.logger.error(f"Failed to publish update: {str(e)}")
            return False

    def _update_remote_manifest(self, file_path, checksum):
        """Update remote manifest with new file information"""
        try:
            if not self.remote_url.startswith('file://'):
                return False

            manifest_path = Path(unquote(urlparse(self.remote_url).path)) / "manifest.json"

            # Load existing manifest
            if manifest_path.exists():
                with open(manifest_path) as f:
                    manifest = json.load(f)
            else:
                manifest = {"version": "0.0.0", "files": {}}

            # Update file information
            manifest["files"][str(file_path)] = {
                "checksum": checksum,
                "updated_at": os.path.getmtime(self.base_dir / file_path)
            }

            # Atomic write using temporary file
            temp_path = manifest_path.with_suffix('.tmp')
            with open(temp_path, 'w') as f:
                json.dump(manifest, f, indent=2)
            temp_path.replace(manifest_path)

            return True

        except Exception as e:
            self.logger.error(f"Failed to update remote manifest: {str(e)}")
            return False

    def check_updates(self):
        """Check for available updates"""
        try:
            remote_manifest = self._fetch_remote_manifest()
            if not remote_manifest:
                return []

            local_manifest = self._get_local_manifest()
            changes = []

            # Check for new or modified files
            for file_path, remote_info in remote_manifest.get("files", {}).items():
                if file_path not in local_manifest.get("files", {}) or \
                   local_manifest["files"][file_path]["checksum"] != remote_info["checksum"]:
                    changes.append({
                        "path": file_path,
                        "type": "modified",
                        "checksum": remote_info["checksum"]
                    })

            # Check for deleted files
            for file_path in local_manifest.get("files", {}):
                if file_path not in remote_manifest.get("files", {}):
                    changes.append({
                        "path": file_path,
                        "type": "deleted"
                    })

            return changes

        except Exception as e:
            self.logger.error(f"Failed to check updates: {str(e)}")
            return []

    def _fetch_remote_manifest(self):
        """Fetch remote version manifest"""
        try:
            if self.remote_url.startswith('file://'):
                manifest_path = Path(unquote(urlparse(self.remote_url).path)) / "manifest.json"
                if manifest_path.exists():
                    with open(manifest_path) as f:
                        return json.load(f)
                return {"version": "0.0.0", "files": {}}
            else:
                response = requests.get(f"{self.remote_url}/manifest.json")
                response.raise_for_status()
                return response.json()

        except Exception as e:
            self.logger.error(f"Error fetching manifest: {str(e)}")
            return None

    def _get_local_manifest(self):
        """Get local version manifest"""
        try:
            manifest_path = self.base_dir / ".redx_manifest.json"
            if manifest_path.exists():
                with open(manifest_path) as f:
                    return json.load(f)
            return {"version": "0.0.0", "files": {}}

        except Exception as e:
            self.logger.error(f"Failed to load local manifest: {str(e)}")
            return {"version": "0.0.0", "files": {}}

    def apply_updates(self, changes):
        """Apply remote updates"""
        try:
            manifest = self._get_local_manifest()
            success = True

            for change in changes:
                if change["type"] == "deleted":
                    file_path = self.base_dir / change["path"]
                    if file_path.exists():
                        file_path.unlink()
                        manifest["files"].pop(change["path"], None)
                        self.logger.info(f"Deleted {change['path']}")
                else:
                    if not self._sync_file(change["path"]):
                        success = False
                        break
                    self.logger.info(f"Updated {change['path']}")

            if success:
                # Update local manifest using atomic write
                manifest_path = self.base_dir / ".redx_manifest.json"
                temp_path = manifest_path.with_suffix('.tmp')
                with open(temp_path, 'w') as f:
                    json.dump(manifest, f, indent=2)
                temp_path.replace(manifest_path)
                return True

            return False

        except Exception as e:
            self.logger.error(f"Failed to apply updates: {str(e)}")
            return False

    def _sync_file(self, file_path):
        """Sync file between remote and local"""
        try:
            if self.remote_url.startswith('file://'):
                remote_base = Path(unquote(urlparse(self.remote_url).path)).resolve()
                src_path = remote_base / file_path
                dst_path = (self.base_dir / file_path).resolve()

                if not src_path.exists():
                    self.logger.error(f"Source file not found: {src_path}")
                    return False

                # Create destination directory if needed
                dst_path.parent.mkdir(parents=True, exist_ok=True)

                # Copy file atomically using temporary file
                temp_path = dst_path.with_suffix('.tmp')
                shutil.copy2(src_path, temp_path)
                temp_path.replace(dst_path)

                # Update local manifest
                manifest = self._get_local_manifest()
                manifest["files"][file_path] = {
                    "checksum": calculate_checksum(dst_path),
                    "updated_at": os.path.getmtime(dst_path)
                }

                # Save updated manifest atomically
                manifest_path = self.base_dir / ".redx_manifest.json"
                temp_path = manifest_path.with_suffix('.tmp')
                with open(temp_path, 'w') as f:
                    json.dump(manifest, f, indent=2)
                temp_path.replace(manifest_path)

                return True

            else:
                response = requests.get(f"{self.remote_url}/{file_path}", stream=True)
                response.raise_for_status()

                dst_path = self.base_dir / file_path
                dst_path.parent.mkdir(parents=True, exist_ok=True)

                # Download to temporary file first
                temp_path = dst_path.with_suffix('.tmp')
                with open(temp_path, 'wb') as f:
                    for chunk in response.iter_content(chunk_size=8192):
                        f.write(chunk)

                # Move temporary file to final location atomically
                temp_path.replace(dst_path)
                return True

        except Exception as e:
            self.logger.error(f"Failed to sync file {file_path}: {str(e)}")
            return False