"""
Enhanced rollback functionality for RedXSecureUpdater with recovery points
and automatic verification
"""
import os
import json
import shutil
import logging
import hashlib
from pathlib import Path
from datetime import datetime
from typing import Optional, Dict, List

class RedXRollback:
    def __init__(self, directory: str):
        # Convert to absolute path to avoid relative path issues
        self.directory = Path(directory).resolve()
        self.backup_dir = self.directory / ".redx_backups"
        self.recovery_dir = self.directory / ".redx_recovery"
        self.state_file = self.directory / ".redx_state.json"
        self.logger = logging.getLogger(__name__)
        self._initialize_state()
        self._initialize_dirs()

    def _initialize_state(self):
        """Initialize state file if it doesn't exist"""
        try:
            if not self.state_file.exists():
                self.directory.mkdir(parents=True, exist_ok=True)
                initial_state = {
                    "version": "0.1.0",
                    "last_update": datetime.now().isoformat(),
                    "files": {}
                }
                with open(self.state_file, 'w') as f:
                    json.dump(initial_state, f, indent=2)
                self.logger.info(f"Created initial state file at {self.state_file}")
        except Exception as e:
            self.logger.error(f"Failed to initialize state file: {str(e)}")
            raise

    def _initialize_dirs(self):
        """Initialize backup and recovery directories"""
        try:
            self.backup_dir.mkdir(parents=True, exist_ok=True)
            self.recovery_dir.mkdir(parents=True, exist_ok=True)
            self.logger.debug("Initialized backup and recovery directories")
        except Exception as e:
            self.logger.error(f"Failed to initialize directories: {str(e)}")
            raise

    def create_recovery_point(self, description: str = "") -> Optional[str]:
        """Create a verified recovery point with description"""
        try:
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            recovery_path = self.recovery_dir / timestamp
            recovery_path.mkdir(parents=True, exist_ok=True)

            # Copy all non-hidden files
            checksums = {}
            for item in self.directory.rglob("*"):
                if item.is_file() and not any(p.startswith('.') for p in item.parts):
                    rel_path = item.relative_to(self.directory)
                    backup_file = recovery_path / rel_path
                    backup_file.parent.mkdir(parents=True, exist_ok=True)
                    shutil.copy2(item, backup_file)
                    checksums[str(rel_path)] = self._calculate_checksum(backup_file)

            # Save metadata with checksums
            metadata = {
                "timestamp": timestamp,
                "description": description,
                "version": self._get_current_version(),
                "checksums": checksums,
                "files": [str(f.relative_to(recovery_path)) 
                         for f in recovery_path.rglob("*") 
                         if f.is_file()]
            }

            with open(recovery_path / "recovery_metadata.json", 'w') as f:
                json.dump(metadata, f, indent=2)

            self.logger.info(f"Created recovery point: {timestamp} - {description}")
            return timestamp

        except Exception as e:
            self.logger.error(f"Recovery point creation failed: {str(e)}")
            return None

    def create_backup(self) -> Optional[str]:
        """Create an automatic backup"""
        try:
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            backup_path = self.backup_dir / timestamp
            backup_path.mkdir(parents=True, exist_ok=True)

            # Copy all files and calculate checksums
            checksums = {}
            for item in self.directory.rglob("*"):
                if item.is_file() and not any(p.startswith('.') for p in item.parts):
                    rel_path = item.relative_to(self.directory)
                    backup_file = backup_path / rel_path
                    backup_file.parent.mkdir(parents=True, exist_ok=True)
                    shutil.copy2(item, backup_file)
                    checksums[str(rel_path)] = self._calculate_checksum(backup_file)

            metadata = {
                "timestamp": timestamp,
                "version": self._get_current_version(),
                "checksums": checksums,
                "files": [str(f.relative_to(backup_path)) 
                         for f in backup_path.rglob("*") 
                         if f.is_file()]
            }

            with open(backup_path / "backup_metadata.json", 'w') as f:
                json.dump(metadata, f, indent=2)

            self.logger.info(f"Created backup: {timestamp}")
            return timestamp

        except Exception as e:
            self.logger.error(f"Backup creation failed: {str(e)}")
            return None

    def restore_recovery_point(self, timestamp: str) -> bool:
        """Restore to a specific recovery point"""
        try:
            points = self.list_recovery_points()
            target_point = next(
                (p for p in points if p["timestamp"] == timestamp),
                None
            )

            if not target_point:
                self.logger.error(f"Recovery point {timestamp} not found")
                return False

            # Create backup before recovery
            self.create_backup()

            # Restore from recovery point
            recovery_path = Path(target_point["path"])

            # Verify files before restoration
            if not self._verify_recovery_point(recovery_path):
                self.logger.error("Recovery point verification failed")
                return False

            for item in recovery_path.rglob("*"):
                if item.is_file() and item.name != "recovery_metadata.json":
                    rel_path = item.relative_to(recovery_path)
                    target_path = self.directory / rel_path
                    target_path.parent.mkdir(parents=True, exist_ok=True)
                    shutil.copy2(item, target_path)

            self.logger.info(f"Restored to recovery point: {timestamp}")
            return True

        except Exception as e:
            self.logger.error(f"Recovery point restoration failed: {str(e)}")
            return False

    def _verify_recovery_point(self, recovery_path: Path) -> bool:
        """Verify recovery point integrity"""
        try:
            metadata_file = recovery_path / "recovery_metadata.json"
            if not metadata_file.exists():
                self.logger.error("Recovery point metadata not found")
                return False

            with open(metadata_file) as f:
                metadata = json.load(f)

            stored_checksums = metadata.get("checksums", {})

            for file_path, stored_checksum in stored_checksums.items():
                full_path = recovery_path / file_path
                if not full_path.exists():
                    self.logger.error(f"Missing file in recovery point: {file_path}")
                    return False

                current_checksum = self._calculate_checksum(full_path)
                if current_checksum != stored_checksum:
                    self.logger.error(f"Checksum mismatch for {file_path}")
                    return False

            return True

        except Exception as e:
            self.logger.error(f"Recovery point verification failed: {str(e)}")
            return False

    def _calculate_checksum(self, file_path: Path) -> str:
        """Calculate SHA-256 checksum of a file"""
        sha256 = hashlib.sha256()
        with open(file_path, 'rb') as f:
            for byte_block in iter(lambda: f.read(4096), b''):
                sha256.update(byte_block)
        return sha256.hexdigest()

    def _get_current_version(self) -> str:
        """Get current version from state file"""
        state_file = self.directory / ".redx_state.json"
        if state_file.exists():
            with open(state_file) as f:
                return json.load(f).get("version", "0.1.0")
        return "0.1.0"  # Default version if state file doesn't exist

    def list_recovery_points(self) -> List[Dict]:
        """List available recovery points"""
        points = []
        if not self.recovery_dir.exists():
            return points

        for point_dir in sorted(self.recovery_dir.iterdir()):
            metadata_file = point_dir / "recovery_metadata.json"
            if metadata_file.exists():
                try:
                    with open(metadata_file) as f:
                        metadata = json.load(f)
                        points.append({
                            "version": metadata["version"],
                            "timestamp": metadata["timestamp"],
                            "description": metadata.get("description", ""),
                            "path": str(point_dir)
                        })
                except Exception as e:
                    self.logger.error(f"Failed to read recovery point metadata: {str(e)}")
                    continue

        return points

    def restore_last_backup(self) -> bool:
        """Restore the most recent backup"""
        try:
            points = self.list_versions()
            if not points:
                self.logger.error("No backups available for restore")
                return False

            latest_version = points[-1]
            return self.rollback_to_version(latest_version["version"])

        except Exception as e:
            self.logger.error(f"Failed to restore last backup: {str(e)}")
            return False

    def list_versions(self) -> List[Dict]:
        """List available backup versions"""
        versions = []
        if not self.backup_dir.exists():
            return versions

        for backup_dir in sorted(self.backup_dir.iterdir()):
            metadata_file = backup_dir / "backup_metadata.json"
            if metadata_file.exists():
                try:
                    with open(metadata_file) as f:
                        metadata = json.load(f)
                        versions.append({
                            "version": metadata["version"],
                            "timestamp": metadata["timestamp"],
                            "path": str(backup_dir)
                        })
                except Exception as e:
                    self.logger.error(f"Failed to read backup metadata: {str(e)}")
                    continue

        return versions

    def rollback_to_version(self, version: str) -> bool:
        """Rollback to a specific version"""
        try:
            versions = self.list_versions()
            target_backup = next(
                (v for v in versions if v["version"] == version), 
                None
            )

            if not target_backup:
                self.logger.error(f"Version {version} not found")
                return False

            # Create a backup of current state before rollback
            self.create_backup()

            # Restore from backup
            backup_path = Path(target_backup["path"])
            for item in backup_path.rglob("*"):
                if item.is_file() and item.name != "backup_metadata.json":
                    rel_path = item.relative_to(backup_path)
                    target_path = self.directory / rel_path
                    target_path.parent.mkdir(parents=True, exist_ok=True)
                    shutil.copy2(item, target_path)

            self.logger.info(f"Rolled back to version {version}")
            return True

        except Exception as e:
            self.logger.error(f"Rollback failed: {str(e)}")
            return False
            
    def restore_backup(self, timestamp: str) -> bool:
        """Restore from a specific backup point"""
        try:
            backup_path = self.backup_dir / timestamp
            if not backup_path.exists():
                self.logger.error(f"Backup {timestamp} not found")
                return False

            metadata_file = backup_path / "backup_metadata.json"
            if not metadata_file.exists():
                self.logger.error("Backup metadata not found")
                return False

            # Verify backup integrity
            with open(metadata_file) as f:
                metadata = json.load(f)
                stored_checksums = metadata.get("checksums", {})

                # Verify all files exist and match checksums
                for file_path, stored_checksum in stored_checksums.items():
                    backup_file = backup_path / file_path
                    if not backup_file.exists():
                        self.logger.error(f"Missing backup file: {file_path}")
                        return False

                    current_checksum = self._calculate_checksum(backup_file)
                    if current_checksum != stored_checksum:
                        self.logger.error(f"Checksum mismatch for {file_path}")
                        return False

            # Restore files from backup
            for file_path in metadata["files"]:
                src = backup_path / file_path
                dst = self.directory / file_path
                dst.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy2(src, dst)

            self.logger.info(f"Successfully restored from backup {timestamp}")
            return True

        except Exception as e:
            self.logger.error(f"Failed to restore backup: {str(e)}")
            return False