"""
Enhanced scheduler functionality for RedXSecureUpdater
Handles timed and scheduled updates with configurable policies
"""
import time
import logging
import threading
import json
from datetime import datetime
from typing import Optional, Dict
from pathlib import Path
import schedule

class RedXScheduler:
    def __init__(self, updater):
        self.updater = updater
        self.logger = logging.getLogger(__name__)
        self.schedule_thread = None
        self.running = False
        self.config_file = Path(updater.directory) / ".redx/redx_schedule.json"
        self._load_config()

    def _load_config(self):
        """Load scheduler configuration"""
        try:
            # Ensure config directory exists
            config_dir = self.config_file.parent
            config_dir.mkdir(parents=True, exist_ok=True)

            if self.config_file.exists():
                with open(self.config_file) as f:
                    self.config = json.load(f)
            else:
                self.config = self._default_config()
                self._save_config()

        except Exception as e:
            self.logger.error(f"Failed to load scheduler config: {str(e)}")
            self.config = self._default_config()

    def _save_config(self):
        """Save scheduler configuration atomically"""
        try:
            temp_file = self.config_file.with_suffix('.tmp')
            with open(temp_file, 'w') as f:
                json.dump(self.config, f, indent=2)
            temp_file.replace(self.config_file)

        except Exception as e:
            self.logger.error(f"Failed to save scheduler config: {str(e)}")

    def _default_config(self) -> Dict:
        """Get default scheduler configuration"""
        return {
            "enabled": False,
            "check_interval": "1h",
            "update_time": "03:00",
            "max_attempts": 3,
            "retry_delay": 300,  # 5 minutes
            "policies": {
                "force_update": False,
                "notify_only": False,
                "backup_before_update": True,
                "auto_rollback": True
            }
        }

    def start(self):
        """Start the scheduler"""
        try:
            if not self.running:
                self.running = True
                self._configure_schedules()
                self.schedule_thread = threading.Thread(target=self._run_scheduler)
                self.schedule_thread.daemon = True
                self.schedule_thread.start()
                self.logger.info("Update scheduler started")

        except Exception as e:
            self.logger.error(f"Failed to start scheduler: {str(e)}")
            self.running = False
            raise

    def stop(self):
        """Stop the scheduler"""
        try:
            self.running = False
            if self.schedule_thread:
                self.schedule_thread.join(timeout=5)
            schedule.clear()
            self.logger.info("Update scheduler stopped")

        except Exception as e:
            self.logger.error(f"Error stopping scheduler: {str(e)}")

    def _run_scheduler(self):
        """Run the scheduler loop"""
        while self.running:
            try:
                schedule.run_pending()
                time.sleep(1)
            except Exception as e:
                self.logger.error(f"Scheduler loop error: {str(e)}")
                time.sleep(5)  # Delay before retry

    def _configure_schedules(self):
        """Configure scheduled tasks based on config"""
        try:
            schedule.clear()

            if not self.config["enabled"]:
                return

            # Configure update check interval
            interval = self.config["check_interval"]
            if interval.endswith('h'):
                hours = int(interval[:-1])
                schedule.every(hours).hours.do(self._check_updates)
            elif interval.endswith('m'):
                minutes = int(interval[:-1])
                schedule.every(minutes).minutes.do(self._check_updates)

            # Configure daily update time
            schedule.every().day.at(self.config["update_time"]).do(
                self._perform_scheduled_update
            )

        except Exception as e:
            self.logger.error(f"Failed to configure schedules: {str(e)}")

    def _check_updates(self):
        """Check for available updates"""
        try:
            if not self.updater.remote:
                return

            changes = self.updater.remote.check_updates()
            if changes:
                if self.config["policies"]["notify_only"]:
                    self.logger.info(f"Updates available: {len(changes)} changes")
                    if self.updater.notifier:
                        self.updater.notifier.show_update_available(len(changes))
                else:
                    self._handle_update_check(changes)

        except Exception as e:
            self.logger.error(f"Update check failed: {str(e)}")

    def _handle_update_check(self, changes):
        """Handle detected updates based on policies"""
        try:
            if self.config["policies"]["force_update"]:
                self._perform_update(changes)
            else:
                self.logger.info(
                    f"Updates available: {len(changes)} changes. "
                    "Waiting for scheduled update time."
                )

        except Exception as e:
            self.logger.error(f"Failed to handle update check: {str(e)}")

    def _perform_scheduled_update(self):
        """Perform scheduled update with backup and rollback"""
        try:
            if not self.updater.remote:
                return

            backup_created = False
            if self.config["policies"]["backup_before_update"]:
                backup_timestamp = self.updater.rollback.create_backup()
                backup_created = bool(backup_timestamp)
                if not backup_created:
                    raise Exception("Failed to create backup before update")

            changes = self.updater.remote.check_updates()
            if changes:
                if self._perform_update(changes):
                    self.logger.info("Scheduled update completed successfully")
                elif self.config["policies"]["auto_rollback"] and backup_created:
                    self.logger.warning("Update failed, initiating automatic rollback")
                    if not self.updater.rollback.restore_last_backup():
                        self.logger.error("Automatic rollback failed")

        except Exception as e:
            self.logger.error(f"Scheduled update failed: {str(e)}")

    def _perform_update(self, changes) -> bool:
        """Attempt to perform update with retries"""
        if not changes or not self.updater.remote:
            return False

        attempts = 0
        while attempts < self.config["max_attempts"]:
            try:
                if self.updater.remote.apply_updates(changes):
                    return True

            except Exception as e:
                self.logger.error(
                    f"Update attempt {attempts + 1} failed: {str(e)}"
                )

            attempts += 1
            if attempts < self.config["max_attempts"]:
                time.sleep(self.config["retry_delay"])

        return False

    def configure(self, **kwargs) -> bool:
        """Update scheduler configuration"""
        try:
            valid_keys = set(self._default_config().keys())
            updates = {k: v for k, v in kwargs.items() if k in valid_keys}

            if updates:
                self.config.update(updates)
                self._save_config()

                if self.running:
                    self._configure_schedules()

                self.logger.info("Scheduler configuration updated")
                return True

            return False

        except Exception as e:
            self.logger.error(f"Failed to configure scheduler: {str(e)}")
            return False

    def get_config(self) -> Optional[Dict]:
        """Get current scheduler configuration"""
        try:
            return self.config.copy()
        except Exception as e:
            self.logger.error(f"Failed to get scheduler config: {str(e)}")
            return None