"""
Simple update server for hosting application updates
"""
import os
import json
import logging
import hashlib
from pathlib import Path
from http.server import HTTPServer, BaseHTTPRequestHandler
import socketserver
from typing import Dict

class UpdateRequestHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        """Handle update check and file download requests"""
        try:
            if self.path == '/check_update':
                self.send_response(200)
                self.send_header('Content-type', 'application/json')
                self.end_headers()

                # Get current version info from server
                version_info = self.server.get_version_info()
                self.wfile.write(json.dumps(version_info).encode())

            elif self.path.startswith('/download/'):
                file_path = self.path.replace('/download/', '')
                full_path = self.server.app_dir / file_path

                if full_path.exists() and not self._is_path_traversal(file_path):
                    self.send_response(200)
                    self.send_header('Content-type', 'application/octet-stream')
                    self.end_headers()

                    with open(full_path, 'rb') as f:
                        self.wfile.write(f.read())
                else:
                    self.send_error(404, "File not found")
            else:
                self.send_error(404, "Not found")

        except Exception as e:
            self.server.logger.error(f"Error handling request: {str(e)}")
            self.send_error(500, "Internal server error")

    def _is_path_traversal(self, path: str) -> bool:
        """Check for path traversal attempts"""
        norm_path = os.path.normpath(path)
        return norm_path.startswith('..') or norm_path.startswith('/')

    def log_message(self, format, *args):
        """Override to use our logger"""
        self.server.logger.info(f"{self.address_string()} - {format%args}")

class UpdateServer(socketserver.TCPServer):
    def __init__(self, app_dir: str, port: int = 8000):
        """Initialize update server"""
        self.app_dir = Path(app_dir).resolve()
        self.version_file = self.app_dir / '.version'
        self.logger = logging.getLogger('UpdateServer')
        self.logger.setLevel(logging.INFO)
        handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)

        # Initialize version if needed
        self._initialize_version()

        # Allow address reuse
        self.allow_reuse_address = True

        super().__init__(('0.0.0.0', port), UpdateRequestHandler)
        self.logger.info(f"Update server initialized on port {port}")

    def _initialize_version(self):
        """Initialize or create version file"""
        try:
            if not self.version_file.exists():
                self._save_version('1.0.0')
        except Exception as e:
            self.logger.error(f"Error initializing version: {str(e)}")

    def _save_version(self, version: str) -> bool:
        """Save version atomically"""
        try:
            temp_file = self.version_file.with_suffix('.tmp')
            with open(temp_file, 'w') as f:
                json.dump({'version': version}, f)
            os.replace(str(temp_file), str(self.version_file))
            return True
        except Exception as e:
            self.logger.error(f"Error saving version: {str(e)}")
            if temp_file.exists():
                try:
                    temp_file.unlink()
                except:
                    pass
            return False

    def _load_version(self) -> str:
        """Load current version"""
        try:
            if not self.version_file.exists():
                self._initialize_version()
                return '1.0.0'
            with open(self.version_file) as f:
                return json.load(f)['version']
        except Exception as e:
            self.logger.error(f"Error loading version: {str(e)}")
            return '1.0.0'

    def increment_version(self) -> str:
        """Increment version number"""
        try:
            current = self._load_version()
            major, minor, patch = map(int, current.split('.'))
            new_version = f"{major}.{minor}.{patch + 1}"
            if self._save_version(new_version):
                self.logger.info(f"Incremented version to {new_version}")
                return new_version
            return current
        except Exception as e:
            self.logger.error(f"Error incrementing version: {str(e)}")
            return current

    def get_version_info(self) -> Dict:
        """Get current version and file information"""
        try:
            version = self._load_version()

            # Scan files and add checksums
            files = {}
            for file_path in self.app_dir.rglob('*'):
                if file_path.is_file() and not self._should_ignore(file_path):
                    rel_path = str(file_path.relative_to(self.app_dir))
                    checksum = self._get_checksum(file_path)
                    if checksum:  # Only include files with valid checksums
                        files[rel_path] = checksum

            return {'version': version, 'files': files}

        except Exception as e:
            self.logger.error(f"Error getting version info: {str(e)}")
            return {'version': '1.0.0', 'files': {}}

    def _should_ignore(self, path: Path) -> bool:
        """Check if file should be ignored"""
        ignore_patterns = [
            '.git',
            '__pycache__',
            '.pyc',
            '.version',
            '.redx',
            'build',
            'dist',
            '.spec',
            '.tmp'
        ]
        return any(pattern in str(path) for pattern in ignore_patterns)

    def _get_checksum(self, file_path: Path) -> str:
        """Calculate file checksum"""
        try:
            hasher = hashlib.sha256()
            with open(file_path, 'rb') as f:
                for chunk in iter(lambda: f.read(4096), b''):
                    hasher.update(chunk)
            return hasher.hexdigest()
        except Exception as e:
            self.logger.error(f"Error calculating checksum for {file_path}: {str(e)}")
            return ""

def start_update_server(app_dir: str, port: int = 8000):
    """Start the update server"""
    try:
        server = UpdateServer(app_dir, port)
        server.logger.info("Starting update server...")
        server.serve_forever()
    except KeyboardInterrupt:
        if server:
            server.logger.info("Shutting down update server...")
            server.shutdown()
    except Exception as e:
        logging.error(f"Error starting server: {str(e)}")
        raise