"""
Simple update client for checking and applying updates
"""
import os
import json
import hashlib
import logging
import requests
from pathlib import Path
from typing import Optional, Dict
import PyInstaller.__main__

class SimpleUpdater:
    def __init__(self, app_dir: str, server_url: str, is_owner: bool = False):
        """
        Initialize simple updater

        Args:
            app_dir: Application directory
            server_url: Update server URL
            is_owner: Whether this is the application owner's instance
        """
        self.app_dir = Path(app_dir).resolve()
        self.server_url = server_url.rstrip('/')
        self.is_owner = is_owner
        self.version_file = self.app_dir / '.version'
        self.is_running = False

        # Setup logging
        self.logger = logging.getLogger('SimpleUpdater')
        handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)
        self.logger.setLevel(logging.INFO)

        # Create necessary directories and files
        self._initialize()

    def _initialize(self):
        """Initialize directories and files"""
        try:
            # Create app directory
            self.app_dir.mkdir(parents=True, exist_ok=True)

            # Create initial version file if needed
            if not self.version_file.exists():
                self._save_version("1.0.0")
        except Exception as e:
            self.logger.error(f"Error initializing: {e}")

    def get_status(self) -> Dict:
        """Get current updater status"""
        try:
            return {
                "version": self._load_version(),
                "is_owner": self.is_owner,
                "server_url": self.server_url,
                "app_dir": str(self.app_dir)
            }
        except Exception as e:
            self.logger.error(f"Error getting status: {str(e)}")
            return {}

    def _load_version(self) -> str:
        """Load current version from file"""
        try:
            if not self.version_file.exists():
                return "1.0.0"
            with open(self.version_file) as f:
                return json.load(f)['version']
        except Exception as e:
            self.logger.error(f"Error loading version: {str(e)}")
            return "1.0.0"

    def _save_version(self, version: str) -> bool:
        """Save version to file atomically"""
        temp_file = None
        try:
            temp_file = self.version_file.with_suffix('.tmp')
            with open(temp_file, 'w') as f:
                json.dump({'version': version}, f)
            os.replace(str(temp_file), str(self.version_file))
            return True
        except Exception as e:
            self.logger.error(f"Error saving version: {str(e)}")
            if temp_file and temp_file.exists():
                try:
                    temp_file.unlink()
                except:
                    pass
            return False

    def check_update(self) -> Optional[Dict]:
        """Check for updates"""
        try:
            # Get current version
            current = self._load_version()
            self.logger.info(f"Current version: {current}")

            # Check server version
            try:
                response = requests.get(f"{self.server_url}/check_update", timeout=5)
                if response.status_code != 200:
                    self.logger.error("Failed to check for updates")
                    return None

                server_info = response.json()
                self.logger.info(f"Server version: {server_info.get('version')}")

                # Compare versions
                if server_info['version'] <= current:
                    self.logger.info("No updates available")
                    return None

                return server_info

            except requests.exceptions.RequestException as e:
                self.logger.error(f"Connection error: {e}")
                return None

        except Exception as e:
            self.logger.error(f"Error checking for updates: {str(e)}")
            return None

    def apply_update(self, update_info: Dict) -> bool:
        """Apply update"""
        try:
            success = True
            self.logger.info("Starting update process...")

            # Download and update files
            for file_path, checksum in update_info['files'].items():
                full_path = self.app_dir / file_path

                # Check if file should be deleted
                if checksum == "DELETE":
                    if full_path.exists():
                        full_path.unlink()
                        self.logger.info(f"Deleted {file_path}")
                    continue

                # Create directories if needed
                full_path.parent.mkdir(parents=True, exist_ok=True)

                # Download file with timeout
                try:
                    response = requests.get(
                        f"{self.server_url}/download/{file_path}",
                        timeout=10
                    )
                    if response.status_code != 200:
                        self.logger.error(f"Failed to download {file_path}")
                        success = False
                        continue

                    # Verify and write file atomically
                    content = response.content
                    if self._verify_checksum(content, checksum):
                        temp_file = full_path.with_suffix('.tmp')
                        with open(temp_file, 'wb') as f:
                            f.write(content)
                        os.replace(str(temp_file), str(full_path))
                        self.logger.info(f"Updated {file_path}")
                    else:
                        self.logger.error(f"Checksum mismatch for {file_path}")
                        success = False
                except Exception as e:
                    self.logger.error(f"Error updating {file_path}: {str(e)}")
                    success = False

            if success:
                # Update version atomically
                if self._save_version(update_info['version']):
                    self.logger.info(f"Successfully updated to version {update_info['version']}")
                    return True
                else:
                    self.logger.error("Failed to update version")
                    return False
            return False

        except Exception as e:
            self.logger.error(f"Error applying update: {str(e)}")
            return False

    def stop(self):
        """Stop the updater and cleanup"""
        try:
            self.is_running = False
            self.logger.info("Updater stopped")
        except Exception as e:
            self.logger.error(f"Error stopping updater: {str(e)}")

    def _verify_checksum(self, content: bytes, expected: str) -> bool:
        """Verify file checksum"""
        hasher = hashlib.sha256()
        hasher.update(content)
        return hasher.hexdigest() == expected

    def build_exe(self) -> bool:
        """Build executable (owner only)"""
        if not self.is_owner:
            return False

        try:
            main_script = next(self.app_dir.glob('*.py'))  # Find first .py file
            exe_name = main_script.stem

            # Build executable with proper spec file
            PyInstaller.__main__.run([
                str(main_script),
                '--onefile',
                '--clean',
                '--noconfirm',
                f'--name={exe_name}',
                f'--distpath={self.app_dir / "dist"}'
            ])

            self.logger.info(f"Built executable: {exe_name}")
            return True

        except Exception as e:
            self.logger.error(f"Error building executable: {str(e)}")
            return False


def check_and_update(app_dir: str, server_url: str) -> bool:
    """
    Check for and apply updates at startup

    Args:
        app_dir: Application directory
        server_url: Update server URL

    Returns:
        Whether any updates were applied
    """
    try:
        updater = SimpleUpdater(app_dir, server_url)
        update_info = updater.check_update()

        if update_info:
            updater.logger.info("Update available, applying...")
            return updater.apply_update(update_info)

        return False

    except Exception as e:
        logging.error(f"Error in update process: {str(e)}")
        return False