"""
Enhanced updater system focused on save-triggered updates with safe executable updating
"""
import os
import json
import shutil
import hashlib
import logging
import time
from pathlib import Path
from datetime import datetime
from watchdog.events import FileSystemEventHandler, FileSystemEvent
from watchdog.observers import Observer
from typing import Dict, Optional, Set
import PyInstaller.__main__
from .version_control import VersionControl

class SaveEventHandler(FileSystemEventHandler):
    def __init__(self, callback):
        self.callback = callback
        self.last_save = 0
        self.cooldown = 0.5  # Minimum time between saves

    def on_modified(self, event):
        if event.is_directory:
            return

        current_time = time.time()
        if current_time - self.last_save >= self.cooldown:
            self.last_save = current_time
            self.callback(event)

class SimpleUpdater:
    def __init__(self, app_dir: str, updates_dir: str = None, 
                 is_publisher: bool = False, main_script: str = None):
        self.app_dir = Path(app_dir).resolve()
        self.updates_dir = Path(updates_dir or 'updates').resolve()
        self.is_publisher = is_publisher
        self.main_script = main_script
        self.exe_name = Path(main_script).stem if main_script else None

        # Initialize version control
        self.version_control = VersionControl(str(self.app_dir))

        # Setup logging
        self.logger = logging.getLogger('SimpleUpdater')
        handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)
        self.logger.setLevel(logging.INFO)

        # Create directories
        self.updates_dir.mkdir(parents=True, exist_ok=True)
        (self.updates_dir / 'dist').mkdir(parents=True, exist_ok=True)

        # Setup file watching for publisher
        self.observer = None
        if is_publisher:
            self.start_watching()

    def start_watching(self):
        """Start watching for file changes"""
        if self.observer:
            return

        try:
            self.observer = Observer()
            handler = SaveEventHandler(self._handle_save)
            self.observer.schedule(handler, str(self.app_dir), recursive=True)
            self.observer.start()
            self.logger.info("Started watching for file saves")
        except Exception as e:
            self.logger.error(f"Failed to start watching: {e}")
            raise

    def _handle_save(self, event: FileSystemEvent):
        """Handle file save event"""
        try:
            # Get current files state
            current_files = self._scan_files()

            # Increment version number
            new_version = self.version_control.increment_version()

            # Create update package
            if self._create_update_package(current_files, new_version):
                self.logger.info(f"Created update package for version {new_version}")

                # Build executable if main script is provided
                if self.is_publisher and self.main_script and self.exe_name:
                    if self._build_exe(new_version):
                        self.logger.info(f"Built and published executable for version {new_version}")
                    else:
                        self.logger.error("Failed to build executable")

        except Exception as e:
            self.logger.error(f"Failed to handle save: {e}")

    def _build_exe(self, version: str) -> bool:
        """Build executable with minimal reliable process"""
        try:
            # Setup build and dist directories
            dist_dir = self.updates_dir / 'dist'
            dist_dir.mkdir(parents=True, exist_ok=True)

            # Build executable with minimal options
            self.logger.info(f"Building executable for version {version}")
            PyInstaller.__main__.run([
                str(self.app_dir / self.main_script),
                '--onefile',
                '--clean',
                '--noconfirm',
                f'--name={self.exe_name}'
            ])

            # Move the built executable to version-specific directory
            source_exe = Path('dist') / f"{self.exe_name}{'.exe' if os.name == 'nt' else ''}"
            if not source_exe.exists():
                raise Exception(f"PyInstaller failed to create executable at {source_exe}")

            version_dir = dist_dir / version
            version_dir.mkdir(parents=True, exist_ok=True)
            target_exe = version_dir / source_exe.name

            # Move executable to version directory
            shutil.move(str(source_exe), str(target_exe))
            os.chmod(str(target_exe), 0o755)  # Make executable

            # Update manifest
            manifest = {
                'version': version,
                'exe_path': f'dist/{version}/{source_exe.name}'
            }

            with open(self.updates_dir / 'exe_manifest.json', 'w') as f:
                json.dump(manifest, f, indent=2)

            # Cleanup build artifacts
            if Path('build').exists():
                shutil.rmtree('build')
            if Path('dist').exists():
                shutil.rmtree('dist')
            for spec_file in Path('.').glob('*.spec'):
                spec_file.unlink()

            self.logger.info(f"Successfully built executable version {version}")
            return True

        except Exception as e:
            self.logger.error(f"Failed to build executable: {str(e)}")
            # Cleanup on failure
            for path in ['build', 'dist']:
                if Path(path).exists():
                    shutil.rmtree(path)
            for spec_file in Path('.').glob('*.spec'):
                spec_file.unlink()
            return False

    def _scan_files(self) -> Dict[str, str]:
        """Scan directory for files and their checksums"""
        files = {}
        try:
            for file_path in self.app_dir.rglob('*'):
                if file_path.is_file() and not self._should_ignore(file_path):
                    rel_path = str(file_path.relative_to(self.app_dir))
                    checksum = self._get_checksum(file_path)
                    if checksum:
                        files[rel_path] = checksum
        except Exception as e:
            self.logger.error(f"Failed to scan files: {e}")
        return files

    def _should_ignore(self, path: Path) -> bool:
        """Check if file should be ignored"""
        ignore_patterns = [
            '.git',
            '__pycache__',
            '.pyc',
            '.pyo',
            '.tmp',
            '.bak',
            'update_state.json',
            'update_manifest.json',
            '.updates',
            '.redx',
            'build',
            'dist',
            '.spec'
        ]
        return any(pattern in str(path).lower() for pattern in ignore_patterns)

    def _get_checksum(self, file_path: Path) -> Optional[str]:
        """Calculate file checksum"""
        try:
            hasher = hashlib.sha256()
            with open(file_path, 'rb') as f:
                for chunk in iter(lambda: f.read(4096), b''):
                    hasher.update(chunk)
            return hasher.hexdigest()
        except Exception as e:
            self.logger.error(f"Failed to calculate checksum: {e}")
            return None

    def _create_update_package(self, files: Dict[str, str], version: str) -> bool:
        """Create update package"""
        try:
            # Create manifest
            manifest = {
                'version': version,
                'timestamp': datetime.now().isoformat(),
                'files': files
            }

            # Save manifest
            with open(self.updates_dir / 'update_manifest.json', 'w') as f:
                json.dump(manifest, f, indent=2)

            # Copy files
            for rel_path in files:
                src = self.app_dir / rel_path
                dst = self.updates_dir / rel_path

                if not src.exists():
                    continue

                dst.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy2(src, dst)

            return True

        except Exception as e:
            self.logger.error(f"Failed to create update package: {e}")
            return False

    def check_for_updates(self) -> bool:
        """Check for and apply updates"""
        if self.is_publisher:
            return False

        try:
            updated = False
            current_version = self.version_control.get_version()

            # Check for code updates
            manifest_file = self.updates_dir / 'update_manifest.json'
            if manifest_file.exists():
                with open(manifest_file) as f:
                    manifest = json.load(f)

                if manifest['version'] > current_version:
                    if self._apply_updates(manifest):
                        updated = True

            # Check for executable updates
            exe_manifest = self.updates_dir / 'exe_manifest.json'
            if exe_manifest.exists() and self.exe_name:
                with open(exe_manifest) as f:
                    exe_info = json.load(f)

                if exe_info['version'] > current_version:
                    if self._update_executable(exe_info):
                        updated = True

            return updated

        except Exception as e:
            self.logger.error(f"Failed to check updates: {e}")
            return False

    def _apply_updates(self, manifest: dict) -> bool:
        """Apply code updates"""
        try:
            # Update files
            for rel_path, checksum in manifest['files'].items():
                src = self.updates_dir / rel_path
                dst = self.app_dir / rel_path

                if not src.exists():
                    continue

                dst.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy2(src, dst)

            # Update version
            self.version_control.set_version(manifest['version'])
            self.logger.info(f"Updated to version {manifest['version']}")
            return True

        except Exception as e:
            self.logger.error(f"Failed to apply updates: {e}")
            return False

    def _update_executable(self, manifest: dict) -> bool:
        """Update executable"""
        try:
            new_exe = self.updates_dir / manifest['exe_path']
            if not new_exe.exists():
                return False

            current_exe = self.app_dir / f"{self.exe_name}{'.exe' if os.name == 'nt' else ''}"

            # Simple replace with backup
            if current_exe.exists():
                backup = current_exe.with_suffix('.bak')
                shutil.copy2(current_exe, backup)

            shutil.copy2(new_exe, current_exe)
            return True

        except Exception as e:
            self.logger.error(f"Failed to update executable: {e}")
            return False

    def stop(self):
        """Stop file watching"""
        if self.observer:
            try:
                self.observer.stop()
                self.observer.join()
                self.logger.info("Stopped watching for changes")
            except Exception as e:
                self.logger.error(f"Failed to stop watching: {e}")