"""
Utility functions for RedXSecureUpdater with robust file operations
"""
import os
import json
import hashlib
import logging
from pathlib import Path
from typing import Dict, Any, Optional
from datetime import datetime

logger = logging.getLogger(__name__)

def calculate_checksum(file_path: Path) -> str:
    """Calculate SHA-256 checksum of a file"""
    sha256 = hashlib.sha256()
    try:
        with open(file_path, 'rb') as f:
            for byte_block in iter(lambda: f.read(4096), b''):
                sha256.update(byte_block)
        return sha256.hexdigest()
    except Exception as e:
        logger.error(f"Error calculating checksum for {file_path}: {e}")
        return ""

def atomic_write(file_path: Path, content: Any) -> bool:
    """Write content to file atomically"""
    temp_path = file_path.with_suffix('.tmp')
    try:
        # Create parent directory if it doesn't exist
        file_path.parent.mkdir(parents=True, exist_ok=True)

        # Write to temporary file first
        with open(temp_path, 'w') as f:
            if isinstance(content, (dict, list)):
                json.dump(content, f, indent=2)
            else:
                f.write(str(content))

        # Atomic replace
        temp_path.replace(file_path)
        return True

    except Exception as e:
        logger.error(f"Error writing file {file_path}: {e}")
        if temp_path.exists():
            try:
                temp_path.unlink()
            except Exception:
                pass
        return False

def safe_read_json(file_path: Path) -> Optional[Dict]:
    """Safely read JSON file"""
    try:
        if not file_path.exists():
            return None
        with open(file_path) as f:
            return json.load(f)
    except Exception as e:
        logger.error(f"Error reading JSON file {file_path}: {e}")
        return None

def should_track_file(file_path: Path) -> bool:
    """Determine if file should be tracked"""
    return (
        file_path.is_file() and
        not any(p.startswith('.') for p in file_path.parts) and
        not file_path.name.startswith('~') and
        not file_path.suffix in ['.pyc', '.pyo', '.tmp'] and
        '__pycache__' not in file_path.parts
    )

def get_file_info(file_path: Path) -> Dict:
    """Get file information including size and modified time"""
    try:
        stat = file_path.stat()
        return {
            "size": stat.st_size,
            "modified": datetime.fromtimestamp(stat.st_mtime).isoformat(),
            "checksum": calculate_checksum(file_path)
        }
    except Exception as e:
        logger.error(f"Error getting file info for {file_path}: {e}")
        return {}

def ensure_directory(dir_path: Path, mode: int = 0o755) -> bool:
    """Ensure directory exists with proper permissions"""
    try:
        if not dir_path.exists():
            dir_path.mkdir(parents=True, exist_ok=True)
        os.chmod(str(dir_path), mode)
        return True
    except Exception as e:
        logger.error(f"Error creating directory {dir_path}: {e}")
        return False

def format_size(size_bytes: int) -> str:
    """Format file size for display"""
    for unit in ['B', 'KB', 'MB', 'GB']:
        if size_bytes < 1024:
            return f"{size_bytes:.1f}{unit}"
        size_bytes /= 1024.0
    return f"{size_bytes:.1f}TB"