"""
Enhanced version control for RedXSecureUpdater with improved initialization
"""
import os
import json
import logging
from pathlib import Path
from datetime import datetime
from typing import Dict, Optional
import re

class VersionControl:
    def __init__(self, directory: str):
        self.directory = Path(directory).resolve()
        self.state_dir = self.directory / '.redx'
        self.state_file = self.state_dir / 'redx_state.json'
        self.logger = logging.getLogger(__name__)

        # Initialize state
        self._initialize_state()

    def _normalize_version(self, version: str) -> str:
        """Normalize version string by removing 'Version ' prefix and validating format"""
        try:
            if not version:
                raise ValueError("Version string cannot be empty")

            # Remove 'Version ' prefix if present and trim whitespace
            version = version.replace("Version ", "").strip()

            # Validate version format (X.Y.Z or X.Y)
            if not re.match(r'^\d+\.\d+(\.\d+)?$', version):
                error_msg = f"Invalid version format: {version}. Expected format: X.Y.Z or X.Y"
                self.logger.error(error_msg)
                raise ValueError(error_msg)

            return version

        except Exception as e:
            self.logger.error(f"Version normalization failed: {str(e)}")
            raise ValueError(str(e))

    def set_version(self, version: str) -> bool:
        """Set specific version with strict validation"""
        try:
            if not version:
                raise ValueError("Version cannot be empty")

            normalized_version = self._normalize_version(version)
            state = self._load_state()
            state["version"] = normalized_version
            state["last_update"] = datetime.now().isoformat()

            # Save state and return success status
            success = self._save_state(state)
            if success:
                self.logger.info(f"Version updated to {normalized_version}")
                return True
            else:
                self.logger.error("Failed to save version state")
                return False

        except ValueError as ve:
            self.logger.error(f"Invalid version format: {str(ve)}")
            raise ValueError(f"Invalid version format: {str(ve)}")

        except Exception as e:
            self.logger.error(f"Failed to set version: {str(e)}")
            return False

    def _initialize_state(self) -> None:
        """Initialize version control state with proper error handling"""
        try:
            # Ensure state directory exists with proper permissions
            os.makedirs(str(self.state_dir), mode=0o755, exist_ok=True)

            if not self.state_file.exists():
                initial_state = {
                    "version": "1.0.0",
                    "last_update": datetime.now().isoformat(),
                    "files": {}
                }
                self._save_state(initial_state)
                self.logger.info("Initialized version control state")

        except Exception as e:
            self.logger.error(f"Failed to initialize state: {str(e)}")
            raise RuntimeError(f"Version control initialization failed: {str(e)}")

    def _save_state(self, state: dict) -> bool:
        """Save state atomically"""
        temp_file = self.state_file.with_suffix('.tmp')
        try:
            # Write to temporary file first
            with open(temp_file, 'w') as f:
                json.dump(state, f, indent=2)

            # Atomic replace
            os.replace(str(temp_file), str(self.state_file))
            return True

        except Exception as e:
            self.logger.error(f"Failed to save state: {str(e)}")
            if temp_file.exists():
                try:
                    temp_file.unlink()
                except Exception:
                    pass
            return False

    def _load_state(self) -> dict:
        """Load current state with proper error handling"""
        try:
            if not self.state_file.exists():
                return {
                    "version": "1.0.0",
                    "last_update": datetime.now().isoformat(),
                    "files": {}
                }

            with open(self.state_file) as f:
                return json.load(f)

        except Exception as e:
            self.logger.error(f"Failed to load state: {str(e)}")
            return {
                "version": "1.0.0",
                "last_update": datetime.now().isoformat(),
                "files": {}
            }

    def get_version(self) -> str:
        """Get current version with fallback"""
        try:
            return self._load_state().get("version", "1.0.0")
        except Exception as e:
            self.logger.error(f"Error getting version: {str(e)}")
            return "1.0.0"

    def check_local_changes(self) -> Dict:
        """Check for local file changes"""
        try:
            current_state = self._load_state()
            return {"version": current_state["version"], "files": current_state["files"]}
        except Exception as e:
            self.logger.error(f"Failed to check local changes: {str(e)}")
            return {"version": "1.0.0", "files": {}}

    def apply_local_changes(self, changes: Dict) -> bool:
        """Apply local changes with validation"""
        try:
            if not isinstance(changes, dict):
                raise ValueError("Invalid changes format")

            current_state = self._load_state()
            current_state["files"].update(changes.get("files", {}))

            # Use provided version or normalize existing one
            new_version = changes.get("version")
            if new_version:
                current_state["version"] = self._normalize_version(new_version)
            current_state["last_update"] = datetime.now().isoformat()

            return self._save_state(current_state)

        except Exception as e:
            self.logger.error(f"Failed to apply local changes: {str(e)}")
            return False